@@ -4,6 +4,7 @@ from typing import Optional, Union
 import numpy as np
 import pandas as pd
 from anndata import AnnData
+from . import logger
 
 def annotate(filename: Union[AnnData,str] = "",
              model: Optional[Union[str, Model]] = None,
@@ -29,7 +30,7 @@ def annotate(filename: Union[AnnData,str] = "",
         Model used to predict the input cells. Default to using the `Immune_All_Low.pkl` model.
         Can be a :class:`~celltypist.models.Model` object that wraps the logistic Classifier and the StandardScaler, the
         path to the desired model file, or the model name.
-        To see all available models and their descriptions, use :func:`~celltypist.models.models_description()`.
+        To see all available models and their descriptions, use :func:`~celltypist.models.models_description`.
     transpose_input
         Whether to transpose the input matrix. Set to `True` if `filename` is provided in a gene-by-cell format.
         (Default: `False`)
@@ -88,6 +89,7 @@ def annotate(filename: Union[AnnData,str] = "",
         if over_clustering in clf.adata.obs:
             over_clustering = clf.adata.obs[over_clustering]
         else:
+            logger.info(f"👀 Did not identify '{over_clustering}' as a cell metadata column, assume it to be a plain text file")
             try:
                 with open(over_clustering, 'rt') as f:
                     over_clustering = [x.strip() for x in f.readlines()]
