@@ -69,7 +69,7 @@ class AnnotationResult():
         df.sort_values(['counts'], ascending=False, inplace=True)
         return df
 
-    def to_adata(self, insert_labels: bool = True, insert_decision: bool = False, insert_prob: bool = False) -> AnnData:
+    def to_adata(self, insert_labels: bool = True, insert_conf: bool = True, insert_decision: bool = False, insert_prob: bool = False, prefix: str = '') -> AnnData:
         """
         Insert the predicted labels, decision or probability matrix, and (if majority voting is done) majority voting results into the AnnData object.
 
@@ -78,28 +78,36 @@ class AnnotationResult():
         insert_labels
             Whether to insert the predicted cell type labels and (if majority voting is done) majority voting-based labels into the AnnData object.
             (Default: `True`)
+        insert_conf
+            Whether to insert the confidence scores (the maximal probability of query cells) into the AnnData object.
+            (Default: `True`)
         insert_decision
             Whether to insert the decision matrix into the AnnData object.
             (Default: `False`)
         insert_prob
             Whether to insert the probability matrix into the AnnData object. This will override the decision matrix even when `insert_decision` is set to `True`.
             (Default: `False`)
+        prefix
+            Prefix for the inserted columns in the AnnData object. Default to no prefix used.
 
         Returns
         ----------
         :class:`~anndata.AnnData`
-            Depending on whether majority voting is done, an :class:`~anndata.AnnData` object with the following columns added to the observation metadata:
+            Depending on whether majority voting is done, an :class:`~anndata.AnnData` object with the following columns (prefixed with `prefix`) added to the observation metadata:
             1) **predicted_labels**, individual prediction outcome for each cell.
             2) **over_clustering**, over-clustering result for the cells.
             3) **majority_voting**, the cell type label assigned to each cell after the majority voting process.
-            4) **name of each cell type**, which represents the decision scores (or probabilities if `insert_prob` is `True`) of a given cell type across cells.
+            4) **conf_score**, the confidence score (maximum probability across all cell types) of each cell.
+            5) **name of each cell type**, which represents the decision scores (or probabilities if `insert_prob` is `True`) of a given cell type across cells.
         """
         if insert_labels:
-            self.adata.obs[self.predicted_labels.columns] = self.predicted_labels
+            self.adata.obs[[f"{prefix}{x}" for x in self.predicted_labels.columns]] = self.predicted_labels
+        if insert_conf:
+            self.adata.obs[f"{prefix}conf_score"] = self.probability_matrix.max(axis=1).values
         if insert_prob:
-            self.adata.obs[self.probability_matrix.columns] = self.probability_matrix
+            self.adata.obs[[f"{prefix}{x}" for x in self.probability_matrix.columns]] = self.probability_matrix
         elif insert_decision:
-            self.adata.obs[self.decision_matrix.columns] = self.decision_matrix
+            self.adata.obs[[f"{prefix}{x}" for x in self.decision_matrix.columns]] = self.decision_matrix
         return self.adata
 
     def to_plots(self, folder: str, plot_probability: bool = False, format: str = 'pdf', prefix: str = '') -> None:
@@ -261,24 +269,24 @@ class Classifier():
                 raise ValueError(f"🛑 The input matrix is detected to be a gene-by-cell matrix. Please provide a cell-by-gene matrix or add the input transpose option")
             sc.pp.normalize_total(self.adata, target_sum=1e4)
             sc.pp.log1p(self.adata)
-            self.indata = self.adata.X.copy()
-            self.indata_genes = self.adata.var_names.copy()
-            self.indata_names = self.adata.obs_names.copy()
+            self.indata = self.adata.X
+            self.indata_genes = self.adata.var_names
+            self.indata_names = self.adata.obs_names
         elif isinstance(filename, AnnData) or (isinstance(filename, str) and filename.endswith('.h5ad')):
             self.adata = sc.read(filename) if isinstance(filename, str) else filename
             self.adata.var_names_make_unique()
             if self.adata.X.min() < 0:
                 logger.info("👀 Detected scaled expression in the input data, will try the .raw attribute")
                 try:
-                    self.indata = self.adata.raw.X.copy()
-                    self.indata_genes = self.adata.raw.var_names.copy()
-                    self.indata_names = self.adata.raw.obs_names.copy()
+                    self.indata = self.adata.raw.X
+                    self.indata_genes = self.adata.raw.var_names
+                    self.indata_names = self.adata.raw.obs_names
                 except Exception as e:
                     raise Exception(f"🛑 Fail to use the .raw attribute in the input object. {e}")
             else:
-                self.indata = self.adata.X.copy()
-                self.indata_genes = self.adata.var_names.copy()
-                self.indata_names = self.adata.obs_names.copy()
+                self.indata = self.adata.X
+                self.indata_genes = self.adata.var_names
+                self.indata_names = self.adata.obs_names
             if np.abs(np.expm1(self.indata[0]).sum()-10000) > 1:
                 raise ValueError("🛑 Invalid expression matrix, expect log1p normalized expression to 10000 counts per cell")
         else:
@@ -314,15 +322,14 @@ class Classifier():
         k_x = np.isin(self.indata_genes, self.model.classifier.features)
         logger.info(f"🧬 {k_x.sum()} features used for prediction")
         k_x_idx = np.where(k_x)[0]
-        self.indata = self.indata[:, k_x_idx]
+        #self.indata = self.indata[:, k_x_idx]
         self.indata_genes = self.indata_genes[k_x_idx]
         lr_idx = pd.DataFrame(self.model.classifier.features, columns=['features']).reset_index().set_index('features').loc[self.indata_genes, 'index'].values
 
         logger.info(f"⚖️ Scaling input data")
         means_ = self.model.scaler.mean_[lr_idx]
         sds_ = self.model.scaler.scale_[lr_idx]
-        self.indata = self.indata - means_
-        self.indata = self.indata / sds_
+        self.indata = (self.indata[:, k_x_idx] - means_) / sds_
         self.indata[self.indata > 10] = 10
 
         self.model.classifier.n_features_in_ = lr_idx.size
@@ -359,7 +366,7 @@ class Classifier():
         ----------
         resolution
             Resolution parameter for leiden clustering which controls the coarseness of the clustering.
-            Default to 5, 10, 15 and 20 for datasets with cell numbers less than 5k, 20k, 40k and above, respectively.
+            Default to 5, 10, 15, 20, 25 and 30 for datasets with cell numbers less than 5k, 20k, 40k, 100k, 200k and above, respectively.
 
         Returns
         ----------
@@ -379,8 +386,12 @@ class Classifier():
                 resolution = 10
             elif self.adata.n_obs < 40000:
                 resolution = 15
-            else:
+            elif self.adata.n_obs < 100000:
                 resolution = 20
+            elif self.adata.n_obs < 200000:
+                resolution = 25
+            else:
+                resolution = 30
         logger.info(f"⛓️ Over-clustering input data with resolution set to {resolution}")
         sc.tl.leiden(self.adata, resolution=resolution, key_added='over_clustering')
         oc_column = self.adata.obs.over_clustering
