@@ -50,13 +50,13 @@ def _prepare_data(X, labels, genes, transpose) -> tuple:
         if adata.X.min() < 0:
             logger.info("👀 Detected scaled expression in the input data, will try the .raw attribute")
             try:
-                indata = adata.raw.X.copy()
-                genes = adata.raw.var_names.copy()
+                indata = adata.raw.X
+                genes = adata.raw.var_names
             except Exception as e:
                 raise Exception(f"🛑 Fail to use the .raw attribute in the input object. {e}")
         else:
-            indata = adata.X.copy()
-            genes = adata.var_names.copy()
+            indata = adata.X
+            genes = adata.var_names
         if isinstance(labels, str) and (labels in adata.obs):
             labels = adata.obs[labels]
         else:
@@ -77,18 +77,18 @@ def _prepare_data(X, labels, genes, transpose) -> tuple:
             logger.warn(f"⚠️ Warning: the input file seems not a raw count matrix. The trained model may be biased")
         sc.pp.normalize_total(adata, target_sum=1e4)
         sc.pp.log1p(adata)
-        indata = adata.X.copy()
-        genes = adata.var_names.copy()
+        indata = adata.X
+        genes = adata.var_names
         labels = _to_vector(labels)
     elif isinstance(X, str):
         raise ValueError("🛑 Invalid input. Supported types: .csv, .txt, .tsv, .tab, .mtx, .mtx.gz and .h5ad")
     else:
         logger.info("👀 The input training data is processed as an array-like object")
-        indata = X.copy()
+        indata = X
         if transpose:
             indata = indata.transpose()
         if isinstance(indata, pd.DataFrame):
-            genes = indata.columns.copy()
+            genes = indata.columns
         else:
             if genes is None:
                 raise Exception("🛑 Missing `genes`. Please provide this argument together with the input training data")
@@ -106,8 +106,8 @@ def _LRClassifier(indata, labels, C, solver, max_iter, n_jobs, **kwargs) -> Logi
     elif solver not in ('liblinear', 'lbfgs', 'newton-cg', 'sag', 'saga'):
         raise ValueError(f"🛑 Invalid `solver`, should be one of `liblinear`, `lbfgs`, `newton-cg`, `sag`, and `saga`")
     logger.info(f"🏋️ Training data using logistic regression")
-    if no_cells > 100000:
-        logger.warn(f"⚠️ Warning: it may take a long time to train this dataset with {no_cells} cells, try to decrease `max_iter` if the training does not finish in practical time")
+    if (no_cells > 100000) and (indata.shape[1] > 10000):
+        logger.warn(f"⚠️ Warning: it may take a long time to train this dataset with {no_cells} cells and {indata.shape[1]} genes, try to downsample cells and/or restrict genes to a subset (e.g., hvgs)")
     classifier = LogisticRegression(C = C, solver = solver, max_iter = max_iter, multi_class = 'ovr', n_jobs = n_jobs, **kwargs)
     classifier.fit(indata, labels)
     return classifier
@@ -121,8 +121,8 @@ def _SGDClassifier(indata, labels,
     classifier = SGDClassifier(loss = 'log', alpha = alpha, max_iter = max_iter, n_jobs = n_jobs, **kwargs)
     if not mini_batch:
         logger.info(f"🏋️ Training data using SGD logistic regression")
-        if len(labels) > 100000:
-            logger.warn(f"⚠️ Warning: it may take a long time to train this dataset with {len(labels)} cells, try to decrease `max_iter` if the training does not finish in practical time")
+        if (len(labels) > 100000) and (indata.shape[1] > 10000):
+            logger.warn(f"⚠️ Warning: it may take a long time to train this dataset with {len(labels)} cells and {indata.shape[1]} genes, try to downsample cells and/or restrict genes to a subset (e.g., hvgs)")
         classifier.fit(indata, labels)
     else:
         logger.info(f"🏋️ Training data using mini-batch SGD logistic regression")
@@ -278,13 +278,13 @@ def train(X = None,
     flag = indata.sum(axis = 0) == 0
     if flag.sum() > 0:
         logger.info(f"✂️ {flag.sum()} non-expressed genes are filtered out")
-        indata = indata[:, ~flag]
+        #indata = indata[:, ~flag]
         genes = genes[~flag]
     #scaler
     logger.info(f"⚖️ Scaling input data")
     scaler = StandardScaler()
-    indata = scaler.fit_transform(indata)
-    indata = np.clip(indata, a_min = None, a_max = 10)
+    indata = scaler.fit_transform(indata[:, ~flag] if flag.sum() > 0 else indata)
+    indata[indata > 10] = 10
     #classifier
     if use_SGD:
         classifier = _SGDClassifier(indata = indata, labels = labels, alpha = alpha, max_iter = max_iter, n_jobs = n_jobs, mini_batch = mini_batch, batch_number = batch_number, batch_size = batch_size, epochs = epochs, balance_cell_type = balance_cell_type, **kwargs)
@@ -299,12 +299,12 @@ def train(X = None,
         gene_index = np.unique(gene_index)
         logger.info(f"🧬 {len(gene_index)} features are selected")
         genes = genes[gene_index]
-        indata = indata[:, gene_index]
+        #indata = indata[:, gene_index]
         logger.info(f"🏋️ Starting the second round of training")
         if use_SGD:
-            classifier = _SGDClassifier(indata = indata, labels = labels, alpha = alpha, max_iter = max_iter, n_jobs = n_jobs, mini_batch = mini_batch, batch_number = batch_number, batch_size = batch_size, epochs = epochs, balance_cell_type = balance_cell_type, **kwargs)
+            classifier = _SGDClassifier(indata = indata[:, gene_index], labels = labels, alpha = alpha, max_iter = max_iter, n_jobs = n_jobs, mini_batch = mini_batch, batch_number = batch_number, batch_size = batch_size, epochs = epochs, balance_cell_type = balance_cell_type, **kwargs)
         else:
-            classifier = _LRClassifier(indata = indata, labels = labels, C = C, solver = solver, max_iter = max_iter, n_jobs = n_jobs, **kwargs)
+            classifier = _LRClassifier(indata = indata[:, gene_index], labels = labels, C = C, solver = solver, max_iter = max_iter, n_jobs = n_jobs, **kwargs)
         scaler.mean_ = scaler.mean_[gene_index]
         scaler.var_ = scaler.var_[gene_index]
         scaler.scale_ = scaler.scale_[gene_index]
